/*  
 *
 * Copyright (C) 2000 Panagiotis Issaris <takis@lumumba.luc.ac.be> 
 *
 * This application (Hades) was developed for my thesis at the
 * School for Knowledge Technology (Belgium/The Netherlands)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * $Author: takis $
 * $Date: 2000/11/07 11:09:39 $
 * $Revision: 1.1 $
 *
 */

#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <math.h>
#include <libmpeg3/libmpeg3.h>

#include "MPEG_Movie.h"

MPEG_Movie *
MPEG_Movie_new ()
{
  MPEG_Movie *m;
  m = (MPEG_Movie *) malloc (sizeof (MPEG_Movie));
  m->filename = NULL;
  m->framebuffer = NULL;
  m->rows = NULL;
  m->file = NULL;
  m->frames = 0;
  m->frame_number = 0;
  m->width = 0;
  m->height = 0;
  return m;
}

void
MPEG_Movie_free (MPEG_Movie * m)
{
  /* FIXME */
  if (m->filename != NULL)
    free (m->filename);
  if (m->framebuffer != NULL)
    free (m->framebuffer);
  if (m->file != NULL)
    free (m->file);

}

MPEG_Movie *
MPEG_Movie_load (char *movie_filename)
{
  int i;
  int rs;
  MPEG_Movie *m;

  m = MPEG_Movie_new ();
  m->filename = movie_filename;

  printf ("--- Opening MPEG ---\n");

  printf ("movie: %s\n", m->filename);
  mpeg3_check_sig (m->filename);
  m->file = mpeg3_open (m->filename);

  mpeg3_set_mmx (m->file, TRUE);
  if (m->file == NULL)
    {
      printf ("Error opening file %s.\n", m->filename);
      exit (1);
    }
  else
    printf ("Opening succeeded.\n");
  if (mpeg3_has_video (m->file))
    printf ("MPEG file has video stream.\n");

  m->nr_vstreams = mpeg3_total_vstreams (m->file);
  m->width = mpeg3_video_width (m->file, m->nr_vstreams - 1);
  m->height = mpeg3_video_height (m->file, m->nr_vstreams - 1);
  m->frames = mpeg3_video_frames (m->file, m->nr_vstreams - 1);
  m->framerate = mpeg3_frame_rate (m->file, m->nr_vstreams - 1);
  printf ("Streams: %d\nWidth: %d\nHeight: %d\n",
	  m->nr_vstreams, m->width, m->height);
  printf ("Frames: %d\nFramerate: %f\n", m->frames, m->framerate);

  m->frame_number = 0;
  mpeg3_set_frame (m->file, m->frame_number, m->nr_vstreams - 1);

  m->framebuffer = (unsigned char *) malloc (
					     (m->width) * (m->height) * 3 +
					     4);

  m->rows = (unsigned char *) malloc (sizeof (unsigned char *) * (m->height));

  for (i = 0; i < (m->height); i++)
    m->rows[i] = &(m->framebuffer[i * (m->width) * 3]);

  printf ("Width: %d\nHeight: %d\nVStreams: %d\n",
	  m->width, m->height, m->nr_vstreams);

  printf ("--- End reading MPEG ---\n");
  return m;
}

GdkPixbuf *
MPEG_Movie_get_frame (MPEG_Movie * m)
{
  GdkPixbuf *frame;

  mpeg3_read_frame (m->file,
		    m->rows,
		    0, 0,
		    m->width, m->height,
		    m->width, m->height, MPEG3_RGB888, m->nr_vstreams - 1);
  m->frame_number++;
  frame = gdk_pixbuf_new_from_data (m->framebuffer,
				    GDK_COLORSPACE_RGB, FALSE,
				    8, m->width, m->height, (m->width) * 3,
				    NULL, NULL);
  return frame;
}

void
MPEG_Movie_rewind (MPEG_Movie * m)
{
  m->frame_number = 0;
  mpeg3_set_frame (m->file, 0, m->nr_vstreams - 1);
}

void
MPEG_Movie_set_frame (MPEG_Movie * m, long frame)
{
  m->frame_number = frame;
  mpeg3_set_frame (m->file, frame, m->nr_vstreams - 1);
}
