/*  
 *
 * Copyright (C) 2000 Panagiotis Issaris <takis@lumumba.luc.ac.be> 
 *
 * This application (Hades) was developed for my thesis at the
 * School for Knowledge Technology (Belgium/The Netherlands)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * $Author: takis $
 * $Date: 2000/11/07 11:09:39 $
 * $Revision: 1.1 $
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"

#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <math.h>
#include <libmpeg3/libmpeg3.h>
#include <gnome.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <pthread.h>
#include "gdk_pixbuf_ops.h"
#include "MPEG_Movie.h"

extern fb_t *fb;

extern GdkPixbuf *frame;
extern GdkPixbuf *framez;
extern GdkEvent *event;
extern GdkEventType type;
extern GdkEvent e;

extern MPEG_Movie *mm;

extern guchar *p;
extern char *filename;
extern int mmc;
extern int play;
extern int i;
extern int error_code;
extern int drawn;
extern int zoom;
extern int zoomfactor;
extern int bps, nc, rs;
extern int width, height;
extern int offset_x, offset_y;
extern int window_width, window_height;

extern pthread_t th_p;
extern pthread_mutex_t lock;
extern pthread_cond_t cond;
extern GtkWidget *application;
extern GtkWidget *drawing_area;
extern GtkWidget *buttonPlay, *buttonPause, *buttonStop, *buttonRewind;
extern GtkWidget *buttonZoomIn, *buttonZoomOut;
extern GtkWidget *buttonPanLeft, *buttonPanRight, *buttonPanUp,
  *buttonPanDown;
extern GtkWidget *vbox, *hbox;
extern GtkWidget *hscale;
extern GtkAdjustment *adjustment;
extern GtkWidget *fs;

GList *effects_list;

void
on_ok_button1_enter (GtkButton * button, gpointer user_data)
{
  gtk_widget_hide (fs);
  open_file (gtk_file_selection_get_filename (fs));
}

void
on_cancel_button1_enter (GtkButton * button, gpointer user_data)
{
  gtk_widget_hide (fs);
}

gint expose_cb (GtkWidget * widget, GdkEventExpose * event, gpointer data)
{
  guchar *pixels;
  int rowstride;
  int width, height;
  int j;

  draw_frame (widget);

  return TRUE;
}

/* Destroy handler for the application*/
void
destroy_cb (GtkObject * object, gpointer data)
{
  if (frame != NULL)
    gdk_pixbuf_unref (frame);

  gtk_main_quit ();
}

void
adj_cb (GtkAdjustment * object, gpointer data)
{
  GdkEvent e;
  unsigned int f = (int) (GTK_ADJUSTMENT (object)->value);
  drawn = FALSE;
  MPEG_Movie_set_frame (mm, f);
}

void *
play_thread (void *p)
{
  GList *li;
  GdkEvent e;

  while (1)
    if (play)
      {
	pthread_mutex_lock (&lock);
	if (fb->nrframes < fb->maxframes)
	  {

	    fb->frames[fb->nrframes++] = MPEG_Movie_get_frame (mm);
	    for (li = effects_list; li != NULL; li = li->next)
	      {
		((frame_convertor) li->data) (fb->frames[fb->nrframes - 1]);
	      }
	  }

	pthread_mutex_unlock (&lock);
      }
}

int
on_timeout (gpointer * p)
{
  GdkEvent e;
  int i;

  if (play)
    {
      e.type = GDK_EXPOSE;
      gtk_propagate_event (GTK_WIDGET (p), &e);
      gtk_adjustment_set_value (adjustment, mm->frame_number);
      gtk_range_clear_background (hscale);
    }
}

int
on_idle (gpointer * p)
{
  GdkEvent e;
  int i;

  if (play)
    {
      e.type = GDK_EXPOSE;
      gtk_propagate_event (GTK_WIDGET (p), &e);
    }
}

void
on_buttonplay_clicked (GtkButton * button, gpointer user_data)
{
  play = TRUE;
  printf ("Clicked play button.\n");
}

void
on_buttonpause_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked pause button.\n");
  play = FALSE;
}

void
on_buttonstop_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked stop button.\n");
  MPEG_Movie_rewind (mm);
}

void
on_buttonopen_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked open button.\n");
  fs = create_fileselection1 ();
  gtk_widget_show (fs);
}


void
on_buttonzoomout_clicked (GtkButton * button, gpointer user_data)
{
  switch (zoom)
    {
    case ZOOM_NOT:
      zoom = ZOOM_OUT;
      zoomfactor = 2;
#ifdef RESIZE_ON_ZOOM
      gtk_drawing_area_size (drawing_area, width / 2, height / 2);
      gtk_widget_set_usize (application, -1, -1);
      gtk_widget_set_usize (application, window_width / 2,
			    window_height / 2 + WINDOW_EXTRA);
#endif
      break;
    case ZOOM_IN:
      zoomfactor /= 2;
      if (zoomfactor == 1)
	zoom = ZOOM_NOT;
#ifdef RESIZE_ON_ZOOM
      gtk_drawing_area_size (drawing_area, width / zoomfactor,
			     height / zoomfactor);
      gtk_widget_set_usize (application, -1, -1);
      gtk_widget_set_usize (application, window_width / zoomfactor,
			    window_height / zoomfactor);
#endif
      break;
    case ZOOM_OUT:
      zoomfactor *= 2;
#ifdef RESIZE_ON_ZOOM
      gtk_widget_ref (drawing_area);
      gtk_container_remove (application, drawing_area);
      gtk_drawing_area_size (drawing_area, width / zoomfactor,
			     height / zoomfactor);
      gtk_container_add (application, drawing_area);
      gtk_widget_unref (drawing_area);
      gtk_widget_set_usize (application, -1, -1);
      gtk_widget_set_usize (application, window_width / zoomfactor,
			    window_height / zoomfactor + WINDOW_EXTRA);
#endif
      break;
    }
  printf ("[%d %d][%d %d]\n", window_width, window_height, width, height);
  printf ("Zooming %s %dx\n",
	  ((zoom == 0) ? "not" : ((zoom == 1) ? "in" : "out")), zoomfactor);
}

void
on_buttonzoomin_clicked (GtkButton * button, gpointer user_data)
{
  printf ("on_buttonzoomin_clicked: Trying to zoom in\n");
  switch (zoom)
    {
    case ZOOM_NOT:
      zoom = ZOOM_IN;
      zoomfactor = 2;
#ifdef RESIZE_ON_ZOOM
      gtk_drawing_area_size (drawing_area, width * 2, height * 2);
      gtk_widget_set_usize (application, window_width * 2,
			    window_height * 2 + WINDOW_EXTRA);
#endif
      break;
    case ZOOM_IN:
      zoomfactor *= 2;
#ifdef RESIZE_ON_ZOOM
      gtk_drawing_area_size (drawing_area, width * zoomfactor,
			     height * zoomfactor);
      gtk_widget_set_usize (application, window_width * zoomfactor,
			    window_height * zoomfactor + WINDOW_EXTRA);
#endif
      break;
    case ZOOM_OUT:
      zoomfactor /= 2;
      if (zoomfactor == 1)
	zoom = ZOOM_NOT;
#ifdef RESIZE_ON_ZOOM
      gtk_drawing_area_size (drawing_area, width / zoomfactor,
			     height / zoomfactor);
      gtk_widget_set_usize (application, window_width / zoomfactor,
			    window_height / zoomfactor + WINDOW_EXTRA);
#endif
      break;
    }
  printf ("Zooming %s %dx\n",
	  ((zoom == 0) ? "not" : ((zoom == 1) ? "in" : "out")), zoomfactor);
}

void
on_buttonpanleft_clicked (GtkButton * button, gpointer user_data)
{
  if (offset_x >= PAN_AMOUNT)
    offset_x -= PAN_AMOUNT;
}

void
on_buttonpanright_clicked (GtkButton * button, gpointer user_data)
{

  offset_x += PAN_AMOUNT;
}

void
on_buttonpanup_clicked (GtkButton * button, gpointer user_data)
{
  if (offset_y > PAN_AMOUNT)
    offset_y -= PAN_AMOUNT;
}

void
on_buttonpandown_clicked (GtkButton * button, gpointer user_data)
{

  offset_y += PAN_AMOUNT;
}

void
on_buttoneffectgrayscale_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked grayscale button.\n");
  effects_list = g_list_append (effects_list, gdk_pixbuf_grayscale);
}

void
on_buttoneffectinvert_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked invert button.\n");
  effects_list = g_list_append (effects_list, gdk_pixbuf_invert);
}


void
on_buttoneffectred_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked red button.\n");
  effects_list = g_list_append (effects_list, gdk_pixbuf_red);
}

void
on_buttoneffectgreen_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked green button.\n");
  effects_list = g_list_append (effects_list, gdk_pixbuf_green);
}
void
on_buttoneffectblue_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked blue button.\n");
  effects_list = g_list_append (effects_list, gdk_pixbuf_blue);
}

void
on_buttoneffectno_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked the no effects button.\n");
  g_list_free (effects_list);
  effects_list = NULL;
}

void
on_buttoneffectemboss_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked the emboss effect button.\n");
  effects_list = g_list_append (effects_list, gdk_pixbuf_emboss);
}

void
on_buttoneffectblur_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked the blur effect button.\n");
  effects_list = g_list_append (effects_list, gdk_pixbuf_blur);
}

void
on_buttoneffectsharpen_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked the sharpen effect button.\n");
  effects_list = g_list_append (effects_list, gdk_pixbuf_sharpen);
}

void
on_buttoneffect_edge_detect_clicked (GtkButton * button, gpointer user_data)
{
  printf ("Clicked the edge detection effect button.\n");
  effects_list = g_list_append (effects_list, gdk_pixbuf_edge_detect);
}

void
on_about_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *about;
  about = create_about ();
  gtk_widget_show (about);
}
