/*  
 *
 * Copyright (C) 2000 Panagiotis Issaris <takis@lumumba.luc.ac.be> 
 *
 * This application (Hades) was developed for my thesis at the
 * School for Knowledge Technology (Belgium/The Netherlands)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * $Author: takis $
 * $Date: 2000/11/07 11:09:39 $
 * $Revision: 1.1 $
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "MPEG_Movie.h"

extern fb_t *fb;
extern MPEG_Movie *mm;

static GdkPixbuf *frame;
static GdkPixbuf *framez;
GdkEvent *event;
GdkEventType type;
GdkEvent e;


guchar *p;
char *filename;
int mmc;
int play;
int i;
int error_code;
int drawn;
int zoom;
int zoomfactor;
int bps, nc, rs;
int width, height;
int offset_x, offset_y;
int window_width, window_height;

pthread_t th_p;
pthread_mutex_t lock;
pthread_cond_t cond;

GtkWidget *application;
GtkWidget *drawing_area;
GtkWidget *buttonPlay, *buttonPause, *buttonStop, *buttonRewind;
GtkWidget *buttonOpen;
GtkWidget *buttonZoomIn, *buttonZoomOut;
GtkWidget *buttonPanLeft, *buttonPanRight, *buttonPanUp, *buttonPanDown;
GtkWidget *buttonGrayscale, *buttonInvert;
GtkWidget *buttonRed, *buttonGreen, *buttonBlue;
GtkWidget *buttonEffectsNo;
GtkWidget *buttonEffectEmboss;
GtkWidget *buttonEffectBlur;
GtkWidget *buttonEffectSharpen;
GtkWidget *buttonEffectEdgeDetect;
GtkWidget *vbox, *hbox;
GtkWidget *hscale;
GtkWidget *toolbar, *toolbarEffects;
GtkAdjustment *adjustment;
GtkWidget *fs;
GtkWidget *toolbar_icon;


static GnomeUIInfo file_menu_uiinfo[] = {
  GNOMEUIINFO_MENU_OPEN_ITEM (on_buttonopen_clicked, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM (destroy_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo help_menu_uiinfo[] = {
  GNOMEUIINFO_MENU_ABOUT_ITEM (on_about_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo menubar_uiinfo[] = {
  GNOMEUIINFO_MENU_FILE_TREE (file_menu_uiinfo),
  GNOMEUIINFO_MENU_HELP_TREE (help_menu_uiinfo),
  GNOMEUIINFO_END
};

GtkWidget *
create_application (void)
{
  application = gnome_app_new ("Takis GdkPixbuf Madness", NULL);


  gtk_window_set_title (GTK_WINDOW (application),
			_("Takis GdkPixbuf Movie Madness"));
  gtk_window_set_policy (GTK_WINDOW (application), FALSE, TRUE, TRUE);

  gtk_signal_connect (GTK_OBJECT (application), "destroy",
		      GTK_SIGNAL_FUNC (destroy_cb), NULL);

  gnome_app_create_menus (GNOME_APP (application), menubar_uiinfo);
  toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  gnome_app_add_toolbar (GNOME_APP (application), GTK_TOOLBAR (toolbar),
			 "toolbar",
			 GNOME_DOCK_ITEM_BEH_EXCLUSIVE,
			 GNOME_DOCK_TOP, 1, 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (toolbar), 1);
  gtk_toolbar_set_space_size (GTK_TOOLBAR (toolbar), 16);
  gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_SPACE_LINE);
  gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar), GTK_RELIEF_NONE);

  toolbarEffects =
    gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  gnome_app_add_toolbar (GNOME_APP (application),
			 GTK_TOOLBAR (toolbarEffects), "toolbarEffects",
			 GNOME_DOCK_ITEM_BEH_EXCLUSIVE, GNOME_DOCK_TOP, 4, 0,
			 0);
  gtk_container_set_border_width (GTK_CONTAINER (toolbarEffects), 1);
  gtk_toolbar_set_space_size (GTK_TOOLBAR (toolbarEffects), 16);
  gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbarEffects),
			       GTK_TOOLBAR_SPACE_LINE);
  gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbarEffects),
				 GTK_RELIEF_NONE);


  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gnome_app_set_contents (GNOME_APP (application), vbox);

  drawing_area = gtk_drawing_area_new ();
  gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event",
		      GTK_SIGNAL_FUNC (expose_cb), NULL);

  gtk_widget_show (drawing_area);
  gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_widget_set_usize (hbox, 100, 20);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

  adjustment = gtk_adjustment_new (0, 0, 0, 1, 10, 0);
  hscale = gtk_hscale_new (adjustment);
  gtk_scale_set_value_pos (GTK_SCALE (hscale), GTK_POS_BOTTOM);
  gtk_scale_set_digits (GTK_SCALE (hscale), 0);
  gtk_widget_set_usize (hscale, -2, -2);
  gtk_signal_connect (GTK_OBJECT (adjustment), "value_changed",
		      GTK_SIGNAL_FUNC (adj_cb), NULL);

  gnome_app_add_docked (GNOME_APP (application), hscale, "hscale",
			GNOME_DOCK_ITEM_BEH_EXCLUSIVE,
			GNOME_DOCK_BOTTOM, 2, 0, 0);

  toolbar_icon =
    gnome_stock_pixmap_widget (application, GNOME_STOCK_PIXMAP_OPEN);
  buttonOpen =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Open"),
				_("Open File"), NULL, toolbar_icon, NULL,
				NULL);

  gtk_signal_connect (GTK_OBJECT (buttonOpen), "clicked",
		      GTK_SIGNAL_FUNC (on_buttonopen_clicked), NULL);

  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-play.xpm"));
  buttonPlay =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Play"),
				_("Play File"), NULL, toolbar_icon, NULL,
				NULL);

  gtk_signal_connect (GTK_OBJECT (buttonPlay), "clicked",
		      GTK_SIGNAL_FUNC (on_buttonplay_clicked), NULL);

  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-pause.xpm"));
  buttonPause =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Pause"),
				_("Pause playing"), NULL, toolbar_icon, NULL,
				NULL);

  gtk_signal_connect (GTK_OBJECT (buttonPause), "clicked",
		      GTK_SIGNAL_FUNC (on_buttonpause_clicked), NULL);

  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-stop.xpm"));
  buttonStop =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Stop"),
				_("Stop playing"), NULL, toolbar_icon, NULL,
				NULL);
  gtk_signal_connect (GTK_OBJECT (buttonStop), "clicked",
		      GTK_SIGNAL_FUNC (on_buttonstop_clicked), NULL);

  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-zoomin.xpm"));
  buttonZoomIn =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("+"),
				_("Zoom in"), NULL, toolbar_icon, NULL, NULL);
  gtk_signal_connect (GTK_OBJECT (buttonZoomIn), "clicked",
		      GTK_SIGNAL_FUNC (on_buttonzoomin_clicked), NULL);


  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-zoomout.xpm"));
  buttonZoomOut =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("-"),
				_("Zoom out"), NULL, toolbar_icon, NULL,
				NULL);
  gtk_signal_connect (GTK_OBJECT (buttonZoomOut), "clicked",
		      GTK_SIGNAL_FUNC (on_buttonzoomout_clicked), NULL);

  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-pan_left.xpm"));
  buttonPanLeft =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("<-"),
				_("Pan left"), NULL, toolbar_icon, NULL,
				NULL);
  gtk_signal_connect (GTK_OBJECT (buttonPanLeft), "clicked",
		      GTK_SIGNAL_FUNC (on_buttonpanleft_clicked), NULL);

  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-pan_right.xpm"));
  buttonPanRight =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("->"),
				_("Pan right"), NULL, toolbar_icon, NULL,
				NULL);
  gtk_signal_connect (GTK_OBJECT (buttonPanRight), "clicked",
		      GTK_SIGNAL_FUNC (on_buttonpanright_clicked), NULL);

  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-pan_up.xpm"));
  buttonPanUp =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Up"),
				_("Pan up"), NULL, toolbar_icon, NULL, NULL);
  gtk_signal_connect (GTK_OBJECT (buttonPanUp), "clicked",
		      GTK_SIGNAL_FUNC (on_buttonpanup_clicked), NULL);

  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-pan_down.xpm"));
  buttonPanDown =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Down"),
				_("Pan down"), NULL, toolbar_icon, NULL,
				NULL);
  gtk_signal_connect (GTK_OBJECT (buttonPanDown), "clicked",
		      GTK_SIGNAL_FUNC (on_buttonpandown_clicked), NULL);




  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-effect_none.xpm"));
  buttonEffectsNo =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbarEffects),
				GTK_TOOLBAR_CHILD_BUTTON, NULL,
				_("No effects"), _("No effects"), NULL,
				toolbar_icon, NULL, NULL);
  gtk_signal_connect (GTK_OBJECT (buttonEffectsNo), "clicked",
		      GTK_SIGNAL_FUNC (on_buttoneffectno_clicked), NULL);

  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file
				("hades-effect_grayscale.xpm"));
  buttonGrayscale =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbarEffects),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Gray"),
				_("Grayscale"), NULL, toolbar_icon, NULL,
				NULL);

  gtk_signal_connect (GTK_OBJECT (buttonGrayscale), "clicked",
		      GTK_SIGNAL_FUNC (on_buttoneffectgrayscale_clicked),
		      NULL);



  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file
				("hades-effect_invert.xpm"));
  buttonInvert =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbarEffects),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Invert"),
				_("Invert"), NULL, toolbar_icon, NULL, NULL);
  gtk_signal_connect (GTK_OBJECT (buttonInvert), "clicked",
		      GTK_SIGNAL_FUNC (on_buttoneffectinvert_clicked), NULL);

  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-effect_red.xpm"));
  buttonRed =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbarEffects),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Red"),
				_("Red pass filter"), NULL, toolbar_icon,
				NULL, NULL);
  gtk_signal_connect (GTK_OBJECT (buttonRed), "clicked",
		      GTK_SIGNAL_FUNC (on_buttoneffectred_clicked), NULL);


  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-effect_green.xpm"));
  buttonGreen =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbarEffects),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Green"),
				_("Green pass filter"), NULL, toolbar_icon,
				NULL, NULL);
  gtk_signal_connect (GTK_OBJECT (buttonGreen), "clicked",
		      GTK_SIGNAL_FUNC (on_buttoneffectgreen_clicked), NULL);


  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-effect_blue.xpm"));
  buttonBlue =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbarEffects),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Blue"),
				_("Blue pass filter"), NULL, toolbar_icon,
				NULL, NULL);
  gtk_signal_connect (GTK_OBJECT (buttonBlue), "clicked",
		      GTK_SIGNAL_FUNC (on_buttoneffectblue_clicked), NULL);

  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file
				("hades-effect_emboss.xpm"));
  buttonEffectEmboss =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbarEffects),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Emboss"),
				_("Emboss filter"), NULL, toolbar_icon, NULL,
				NULL);
  gtk_signal_connect (GTK_OBJECT (buttonEffectEmboss), "clicked",
		      GTK_SIGNAL_FUNC (on_buttoneffectemboss_clicked), NULL);


  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file ("hades-effect_blur.xpm"));
  buttonEffectBlur =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbarEffects),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Blur"),
				_("Blur filter"), NULL, toolbar_icon, NULL,
				NULL);
  gtk_signal_connect (GTK_OBJECT (buttonEffectBlur), "clicked",
		      GTK_SIGNAL_FUNC (on_buttoneffectblur_clicked), NULL);



  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file
				("hades-effect_sharpen.xpm"));
  buttonEffectSharpen =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbarEffects),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Sharpen"),
				_("Sharpen filter"), NULL, toolbar_icon, NULL,
				NULL);
  gtk_signal_connect (GTK_OBJECT (buttonEffectSharpen), "clicked",
		      GTK_SIGNAL_FUNC (on_buttoneffectsharpen_clicked), NULL);



  toolbar_icon =
    gnome_pixmap_new_from_file (gnome_pixmap_file
				("hades-effect_edge_detect.xpm"));
  buttonEffectEdgeDetect =
    gtk_toolbar_append_element (GTK_TOOLBAR (toolbarEffects),
				GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Edge"),
				_("Edge detection"), NULL, toolbar_icon, NULL,
				NULL);
  gtk_signal_connect (GTK_OBJECT (buttonEffectEdgeDetect), "clicked",
		      GTK_SIGNAL_FUNC (on_buttoneffect_edge_detect_clicked),
		      NULL);






  gtk_widget_show_all (application);
  return application;
}

GtkWidget *
create_about (void)
{
  const gchar *authors[] = {
    "Panagiotis Issaris <takis@lumumba.luc.ac.be>",
    NULL
  };

  GtkWidget *about;

  about = gnome_about_new ("Hades Movieplayer", "0.1",
			   "(C) 2000 by Panagiotis Issaris",
			   authors,
			   _("A movieplayer using GdkPixbuf"), "logo.xpm");
  gtk_window_set_modal (GTK_WINDOW (about), TRUE);
  return about;
}

GtkWidget *
create_fileselection1 (void)
{
  GtkWidget *fileselection1;
  GtkWidget *ok_button1;
  GtkWidget *cancel_button1;

  fileselection1 = gtk_file_selection_new (_("Select File"));
  gtk_object_set_data (GTK_OBJECT (fileselection1), "fileselection1",
		       fileselection1);
  gtk_container_set_border_width (GTK_CONTAINER (fileselection1), 10);

  ok_button1 = GTK_FILE_SELECTION (fileselection1)->ok_button;
  gtk_object_set_data (GTK_OBJECT (fileselection1), "ok_button1", ok_button1);
  gtk_widget_show (ok_button1);
  GTK_WIDGET_SET_FLAGS (ok_button1, GTK_CAN_DEFAULT);

  cancel_button1 = GTK_FILE_SELECTION (fileselection1)->cancel_button;
  gtk_object_set_data (GTK_OBJECT (fileselection1), "cancel_button1",
		       cancel_button1);
  gtk_widget_show (cancel_button1);
  GTK_WIDGET_SET_FLAGS (cancel_button1, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (ok_button1), "clicked",
		      GTK_SIGNAL_FUNC (on_ok_button1_enter), NULL);
  gtk_signal_connect (GTK_OBJECT (cancel_button1), "clicked",
		      GTK_SIGNAL_FUNC (on_cancel_button1_enter), NULL);


  return fileselection1;
}
